@@ -264,12 +264,22 @@ class TestTagNameMatchRule(SoupTest):
         tag = Tag(**tag_kwargs)
         assert rule.matches_tag(tag) == result
 
+    def test_matches_tag_only_passes_tag_to_function(self):
+        def arg1_must_be_tag(t):
+            if not isinstance(t, Tag):
+                raise ValueError("Non-tag passed into tag name match rule function")
+            return True
+
+        rule = TagNameMatchRule(function=arg1_must_be_tag)
+        assert rule.matches_tag(Tag(name="tagname")) is True
+
 
 # AttributeValueMatchRule and StringMatchRule have the same
 # logic as MatchRule.
 
 
 class TestSoupStrainer(SoupTest):
+
     def test_constructor_string_deprecated_text_argument(self):
         with warnings.catch_warnings(record=True) as w:
             strainer = SoupStrainer(text="text")
@@ -320,6 +330,14 @@ class TestSoupStrainer(SoupTest):
     def _match_function(x):
         pass
 
+    def test_constructor_default(self):
+        # The default SoupStrainer matches all tags, and only tags.
+        strainer = SoupStrainer()
+        [name_rule] = strainer.name_rules
+        assert True == name_rule.present
+        assert 0 == len(strainer.attribute_rules)
+        assert 0 == len(strainer.string_rules)
+
     def test_constructor(self):
         strainer = SoupStrainer(
             "tagname",
